package math.util;

import java.util.*;

/** Adds a bunch of numbers together, where the numbers are
 * represented by logarithms.  It generally follows the
 * function "log_add" on p. 337 of Manning and Schutze's
 * _Foundations of Statistical NLP_, but stores
 * the numbers, and then computes the sum of all of them
 * at once.
 *   Uses "log to the base e" throughout. */
public class LogAdder {

  /** The largest number seen so far that's being added. */
  private double max = Double.NEGATIVE_INFINITY;

  /** Things less than (max - logBig) aren't even stored. */
  private double logBig = 17;    // following the book...

  /** Vector containing Double() objects, which are
   * the logs of the numbers to be added. */
  private Vector v = new Vector();

  /** Constructor. */
  public LogAdder() { }

  /** Constructor allowing a larger degree of sloppiness in
   * the addition.  You should be able to trade off precision
   * for speed, a bit, by tweaking this parameter. */
  public LogAdder(double logBig) {
    this.logBig = logBig;
  }

  /** Add one number to this sum.
   * @param x  the <i>log</i> of the number to add */
  public void add(double x) {

    // possibly recompute the maximum
    if (x > max) max = x;

    // only store x if it's big enough to not be ignored in sum()
    if (x > (max - logBig))
      v.addElement(new Double(x));
  }

  /** Compute the log of the sum.
   * @return  the log of the sum of the numbers whose logarithms
   *   have been given using add(). */
  public double logSum() {
    double s = 0;

    // loop through the Vector of numbers
    for (Enumeration e = v.elements() ; e.hasMoreElements() ; ) {
      Double d = (Double) e.nextElement();
      double x = d.doubleValue();   // XXX ugly iterators and casts
      if (x > (max - logBig))
        s += Math.exp(x - max);

    }

    return Math.log(s) + max;
  }

}

